Class: Taro::Types::RailsParamsType
- Inherits:
-
InputType
- Object
- Struct
- BaseType
- ObjectType
- InputType
- Taro::Types::RailsParamsType
- Defined in:
- lib/taro/types/rails_params_type.rb
Overview
Abstract base class for rails declaration params. Internal use only.
Constant Summary
Constants included from Shared::OpenAPIFormat
Shared::OpenAPIFormat::OPENAPI_INTEGER_FORMATS, Shared::OpenAPIFormat::OPENAPI_NUMBER_FORMATS, Shared::OpenAPIFormat::OPENAPI_STRING_FORMATS
Constants included from Shared::OpenAPIType
Shared::OpenAPIType::OPENAPI_TYPES
Instance Attribute Summary
Attributes inherited from BaseType
Attributes included from Shared::AdditionalProperties
Attributes included from Shared::Deprecation
Instance Method Summary collapse
-
#validate_no_undeclared_params? ⇒ Boolean
Skip validation of base params because they contain rails “additions” like controller, action, routing-related stuff, de-nested values, etc.
Methods inherited from InputType
Methods included from Shared::Fields
Methods included from Shared::ObjectCoercion
#coerce_input, #coerce_response
Methods included from Shared::CustomFieldResolvers
Methods included from Shared::AdditionalProperties
#additional_properties?, #inherited
Methods included from Shared::DerivedTypes
#define_derived_type, #derived_types, map
Methods included from Shared::Description
Methods included from Shared::Equivalence
#equal_properties?, #equivalent?
Methods included from Shared::Name
Methods included from Shared::OpenAPIFormat
#inherited, #openapi_format, #openapi_format=
Methods included from Shared::OpenAPIName
#default_openapi_name, #openapi_name, #openapi_name=
Methods included from Shared::OpenAPIType
#inherited, #openapi_type, #openapi_type=
Methods included from Shared::Rendering
#last_render, #last_render=, #render, #used_in_response
Methods included from Shared::TypeClass
Methods included from Shared::Errors
#coerce_error_message, #input_error, #response_error
Methods included from Shared::Caching
#cached_coerce_response, included
Instance Method Details
#validate_no_undeclared_params? ⇒ Boolean
Skip validation of base params because they contain rails “additions” like controller, action, routing-related stuff, de-nested values, etc.
7 8 9 |
# File 'lib/taro/types/rails_params_type.rb', line 7 def validate_no_undeclared_params? false end |